from si_tests.clients import http_client
import json
import yaml
from si_tests import logger

LOG = logger.logger


class MCCDashboardClient(object):
    token = None

    def __init__(self, dashboard_url,
                 username="writer", password="password",
                 verify=False, token=None):
        self.username = username
        self.password = password
        self.httpclient = http_client.HttpClient(base_url=dashboard_url)
        self.verify = verify
        self.username = username
        self.password = password
        self.httpclient = http_client.HttpClient(base_url=dashboard_url)
        self.token = token
        if not self.token:
            self.login()

    def compose_req_header(self):
        return {"Content-Type": "application/json",
                "Accept": "application/json",
                "Referer": "http://172.19.118.240/projects",
                "Host": "172.19.118.240",
                "Authorization": "Bearer {}".format(self.id_token)}

    def get_resource(self, resource_name, decode=True, raise_on_error=True):
        url = "/{0}".format(resource_name)
        try:
            res = self.httpclient.request(
                url=url, method='GET',
                headers=self.compose_req_header(),
                verify=self.verify,
                raise_on_error=raise_on_error)
            if res.status_code == 401:
                # Try to refresh the token
                self.login()
                res = self.httpclient.request(
                    url=url, method='GET',
                    headers=self.compose_req_header(),
                    verify=self.verify,
                    raise_on_error=raise_on_error)
        except Exception as e:
            LOG.warning(e)
            # Try to refresh token again
            self.login()
            res = self.httpclient.request(
                url=url, method='GET',
                headers=self.compose_req_header(),
                verify=self.verify,
                raise_on_error=raise_on_error)
        LOG.debug(f"Raw response:\n{res.content}")
        if decode:
            result = json.loads(res.content.decode('utf-8'))
            LOG.debug(f"Response:\n{yaml.dump(result)}")
            return result
        else:
            return res.content.decode('utf-8')

    def compose_login_header(self):
        header = {"Content-Type": "application/x-www-form-urlencoded"}
        if self.token:
            header.update({"Authorization": "Bearer {}".format(self.token)})
        return header

    def login(self):
        """Authorize with username/password and get new token"""
        self.token = None
        url = "/auth/realms/iam/protocol/openid-connect/token"

        body_data = {
            'username': self.username,
            'password': self.password,
            'client_id': 'kaas',
            'grant_type': 'password',
            'response_type': 'id_token',
            'scope': 'openid',
        }
        body = '&'.join([f"{k}={v}" for k, v in body_data.items()])

        LOG.debug(f"Login body: {body}")

        res = self.httpclient.request(url=url, method='POST',
                                      headers=self.compose_login_header(),
                                      body=body,
                                      verify=self.verify)
        LOG.debug(f"Login response: {res}")
        decoded_res = json.loads(res.content.decode('utf-8'))
        if "access_token" not in decoded_res:
            raise ValueError("Authentication failed, MCC UI response: {0}"
                             .format(decoded_res))
        self.token = decoded_res["access_token"]
        self.id_token = decoded_res["id_token"]
        LOG.info("MCC UI auth token have been refreshed")
