from si_tests import logger
from keycloak import KeycloakAdmin
from keycloak import KeycloakOpenID
from si_tests import settings
from si_tests.clients import http_client
import json

LOG = logger.logger


class KeycloakAdminClient(object):

    def __init__(self, ip,
                 user=settings.KEYCLOAK_USER,
                 password=settings.KEYCLOAK_PASSWORD,
                 verify=settings.IAM_CACERT, realm='master',
                 client_id='admin-cli', grant_type='password'):
        self.url = "https://" + ip
        self.user = user
        self.password = password
        # realm is master by default
        # self.realm = realm
        # client_id is admin-cli by default
        # self.client_id = client_id
        # grant_type is password by default
        # self.grant_type = grant_type
        self.cacert_path = verify
        self.httpclient = http_client.HttpClient(base_url=self.url)
        self.client = KeycloakAdmin(server_url="{}/auth/".format(self.url),
                                    username=self.user,
                                    password=self.password,
                                    verify=self.cacert_path)
        # switching to iam realm
        self.client.realm_name = 'iam'

    def get_token(self):
        return self.client.token

    def user_create(self, username, password, email=None,
                    firstname="si", lastname="test", enabled=True):
        self.client.create_user({"email": email,
                                 "username": username,
                                 "enabled": enabled,
                                 "firstName": firstname,
                                 "lastName": lastname,
                                 "credentials": [{"value": password,
                                                  "type": "password", }]})

    def user_delete(self, username):
        id = self.client.get_user_id(username)
        self.client.delete_user(id)

    def user_update(self, username, body):
        id = self.client.get_user_id(username)
        self.client.update_user(id, body)

    def user_get(self, username):
        id = self.client.get_user_id(username)
        return self.client.get_user(id)

    def roles_get(self, roles):
        all_roles = self.client.get_realm_roles()
        body = []
        for role in all_roles:
            if role['name'] in roles:
                body.append(role)
        return body

    def compose_header(self):
        return {"Content-Type": "application/json",
                "Authorization": "bearer {}".format(
                    self.get_token()["access_token"]
                )}

    def assign_roles(self, username, roles, servicename='iam'):
        user_id = self.user_get(username)['id']
        roles_list = self.roles_get(roles)
        url = "/auth/admin/realms/{0}/users/{1}/role-mappings/realm".format(
            servicename, user_id
        )
        self.httpclient.request(url=url, method='POST',
                                headers=self.compose_header(),
                                body=json.dumps(roles_list),
                                verify=self.cacert_path)
        return True

    def sync_user(self, username, password):
        # sync from ldap
        raise NotImplementedError("To be implemented")


class KeycloakUserClient(object):

    def __init__(self, ip, user, password,
                 verify=settings.IAM_CACERT, client_id='iam',
                 realm_name='iam', client_secret_key=""):
        self.user = user
        self.password = password
        self.client = KeycloakOpenID(server_url="https://{}/auth/".format(ip),
                                     client_id=client_id,
                                     realm_name=realm_name,
                                     client_secret_key=client_secret_key,
                                     verify=verify)
        # Get WellKnow
        # config_well_know = keycloak_openid.well_know()

        self.token = self.client.token(self.user, self.password)

        # Get Token
    def get_token(self):
        return self.token['access_token']

    def refresh_token(self):
        return self.client.refresh_token(self.token['refresh_token'])

    def get_openid_token(self):
        return self.client.token(self.user, self.password, scope='openid')
