#    Copyright 2017 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations


from si_tests.clients.k8s.base import K8sBaseResource
from si_tests.clients.k8s.base import K8sBaseManager
from si_tests import logger

LOG = logger.logger


class K8sJob(K8sBaseResource):
    resource_type = 'job'

    def _read(self, **kwargs):
        return self._manager.api.read_namespaced_job(
            self.name, self.namespace, **kwargs)

    def _create(self, body, **kwargs):
        return self._manager.api.create_namespaced_job(
            self.namespace, body, **kwargs)

    def _patch(self, body, **kwargs):
        return self._manager.api.patch_namespaced_job(
            self.name, self.namespace, body, **kwargs)

    def _replace(self, body, **kwargs):
        return self._manager.api.replace_namespaced_job(
            self.name, self.namespace, body, **kwargs)

    def _delete(self, **kwargs):
        self._manager.api.delete_namespaced_job(
            self.name, self.namespace, **kwargs)


class K8sJobManager(K8sBaseManager):
    resource_class = K8sJob

    @property
    def api(self):
        return self._cluster.api_batch

    def _list(self, namespace, **kwargs):
        return self.api.list_namespaced_job(namespace, **kwargs)

    def _list_all(self, **kwargs):
        return self.api.list_job_for_all_namespaces(**kwargs)

    def check_jobs_completed(self, target_namespaces=None,
                             excluded_jobs=[], jobs_prefix=''):
        jobs = self.list_raw().to_dict()['items']
        failed_jobs = {}
        for job in jobs:
            name = job['metadata']['name']
            namespace = job['metadata']['namespace']
            status = job.get('status', {})
            if target_namespaces and namespace not in target_namespaces:
                continue
            for one in excluded_jobs:
                if name.startswith(one):
                    continue
            if jobs_prefix and not name.startswith(jobs_prefix):
                continue
            if status.get('active', {}) is not None \
                    or status.get('succeeded', {}) != 1:
                failed_jobs[name] = {'active': status.get('active', {}),
                                     'succeeded': status.get('succeeded', {})}
        if len(failed_jobs) > 0:
            LOG.warning(f"Next jobs did not finish or failed: {failed_jobs}")
            return False
        return True
