import pytest

from si_tests.clients.iam import iam_client
from si_tests.clients.iam import keycloak_client
from si_tests import settings


@pytest.fixture(scope='session')
def keycloak_ip(kaas_manager):
    return kaas_manager.get_keycloak_ip()


@pytest.fixture(scope='session')
def iam_ip_port(kaas_manager):
    return kaas_manager.get_iam_ip_port()


@pytest.fixture(scope='session')
def iam_api(kaas_manager, keycloak_ip, iam_ip_port):
    cacert_path = 'keycloakCA.pem'
    cacert = kaas_manager.get_secret_data('iam-api-secrets',
                                          'kaas', cacert_path)
    with open(cacert_path, 'w') as f:
        f.write(cacert)

    return iam_client.IAMClient(keycloak_ip=keycloak_ip,
                                iam_ip_port=iam_ip_port,
                                cacert_path=cacert_path)


@pytest.fixture(scope='session')
def keycloak_admin_client(kaas_manager, keycloak_ip):
    # 1) we assume Keycloak contains keycloak user in IAM and Master realms
    # by default
    # 2) This user is the only one that can create other users

    pwd = kaas_manager.get_secret_data('iam-api-secrets',
                                       'kaas', 'keycloak_password')

    return keycloak_client.KeycloakAdminClient(ip=keycloak_ip,
                                               user=settings.KEYCLOAK_USER,
                                               password=pwd)
