import yaml

import exec_helpers

from si_tests import settings


class Helm3Manager(object):
    binary = settings.HELM_BINARY_PATH

    def __init__(self, namespace='openstack'):
        self.namespace = namespace

    def run_cmd(self, cmd):
        cmd = " ".join([self.binary, *cmd])
        res = exec_helpers.Subprocess().execute(cmd)
        return (res.stdout_str, res.stderr_str)

    def list(self, args=None):
        args = args or []
        cmd = [
            "list",
            "--namespace",
            self.namespace,
            "-o",
            "json",
            *args,
        ]
        stdout, stderr = self.run_cmd(cmd)
        return yaml.load(stdout)

    def get_release_values(self, name, args=None):
        args = args or []
        cmd = [
            "get",
            "values",
            "--namespace",
            self.namespace,
            name,
            "-o",
            "json",
            *args,
        ]
        stdout, stderr = self.run_cmd(cmd)
        return yaml.load(stdout)

    def get_releases_values(self, args=None):
        args = args or []
        res = {}
        for release in self.list(args):
            name = release['name']
            res[name] = self.get_release_values(name)
        return res
