
from si_tests import logger
import pyipmi
import pyipmi.interfaces

LOG = logger.logger


class IpmiManager(object):
    """Ipmi manager"""

    def interface(self, ipmi_host='', ipmi_user='', ipmi_password=''):
        def establish_session(ipmi, ipmi_host='', ipmi_user='',
                              ipmi_password='', ipmi_port=623):
            ipmi.session.set_session_type_rmcp(host=ipmi_host, port=ipmi_port)
            ipmi.session.set_auth_type_user(username=ipmi_user,
                                            password=ipmi_password)
            ipmi.session.establish()
            return ipmi

        try:
            interface = \
                pyipmi.interfaces.create_interface(interface='rmcp',
                                                   slave_address=0x81,
                                                   host_target_address=0x20)
            ipmi = pyipmi.create_connection(interface)
            ipmi.target = pyipmi.Target(ipmb_address=0x20)
            establish_session(ipmi, ipmi_host, ipmi_user, ipmi_password, 623)
        except pyipmi.errors.CompletionCodeError:
            interface = \
                pyipmi.interfaces.create_interface(interface='ipmitool',
                                                   interface_type='lanplus')
            ipmi = pyipmi.create_connection(interface)
            ipmi.target = pyipmi.Target(ipmb_address=0x20)
            establish_session(ipmi, ipmi_host, ipmi_user, ipmi_password, 623)
        finally:
            return ipmi

    def get_power_on(self, ipmi_host='', ipmi_user='', ipmi_password=''):
        """
        Host power is on?
        return True or False
        """
        ipmi = self.interface(ipmi_host, ipmi_user, ipmi_password)
        chassis_status = ipmi.get_chassis_status()
        return chassis_status.power_on

    def set_power_off(self, ipmi_host='', ipmi_user='', ipmi_password=''):
        """
        Hard type machine power off
        """
        ipmi = self.interface(ipmi_host, ipmi_user, ipmi_password)
        return ipmi.chassis_control_power_down()

    def set_power_on(self, ipmi_host='', ipmi_user='', ipmi_password=''):
        """
        Power on the machine
        """
        ipmi = self.interface(ipmi_host, ipmi_user, ipmi_password)
        return ipmi.chassis_control_power_up()

    def set_power_reset(self, ipmi_host='', ipmi_user='', ipmi_password=''):
        """
        Machine power reset
        """
        ipmi = self.interface(ipmi_host, ipmi_user, ipmi_password)
        return ipmi.chassis_control_hard_reset()
