#    Copyright 2020 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from si_tests import logger
from si_tests import settings

from si_tests.managers import bootstrap_manager

LOG = logger.logger


class TestProvisionOsRegionalClusters():

    def test_deploy_os_regional_clean_seed(self, kaas_manager,
                                           openstack_client,
                                           check_sg, show_step):
        """Deploy Ost Regional cluster on KaaS mgmt

        Scenario:
            1. Prepare seed node to run KaaS bootstrap under user 'regional'
            2. Check bootstrap templates for regional cluster
            3. Copy the management cluster kubeconfig
               from 'ubuntu' user to 'regional'
            4. Using region variables metadata and path to management
               kubeconfig, prepare env and run bootsrap.sh deploy_regional
            5. Copy regional bootstrap artifacts under 'ubuntu' user
            6. Check result of deployment
        """

        bootstrap = bootstrap_manager.BootstrapManager()

        show_step(1)
        bootstrap.step_003_prepare_seed_node(openstack_client)

        if settings.SKIP_BOOTSTRAP_TEMPLATES_CHECK:
            LOG.warning("Skip bootstrap templates check due "
                        "SKIP_BOOTSTRAP_TEMPLATES_CHECK flag is set")
        else:
            show_step(2)
            bootstrap.check_templates(openstack_client)

        show_step(3)
        # copy mgmt kubeconfig from default bootstrap user space
        regional_cluster_bootstrap_userspace = settings.SEED_SSH_LOGIN
        regional_home = f"/home/{regional_cluster_bootstrap_userspace}"
        mgmt_home = f"/home/{settings.KAAS_MGMT_CLUSTER_BOOTSTRAP_USERSPACE}"

        bootstrap.copy_user_file(
          mgmt_home + "/kubeconfig",
          regional_cluster_bootstrap_userspace,
          regional_home + "/kubeconfig",
          openstack_client)

        show_step(4)
        bootstrap.step_004_deploy_kaas_regional(openstack_client,
                                                region=settings.OS_REGION)

        show_step(5)
        # copy regional kubeconfig and ssh key into mgmt home boostrap dir
        bootstrap.store_cluster_artifacts(
                openstack_client=openstack_client,
                cluster_namespace="default",
                cluster_name=settings.REGIONAL_CLUSTER_NAME,
                cluster_kubeconfig_path="{}/kubeconfig-{}"
                .format(regional_home, settings.REGIONAL_CLUSTER_NAME),
                cluster_ssh_key_path="{}/{}/ssh_key"
                .format(regional_home, settings.KAAS_BOOTSTRAP_TARGET_DIR))

        show_step(6)
        bootstrap.step_006_wait_for_pods()
        bootstrap.step_006_postdeployment_checks(openstack_client)

        ns = kaas_manager.get_namespace(settings.REGION_NAMESPACE)
        regional_cluster = ns.get_cluster(settings.REGIONAL_CLUSTER_NAME)
        regional_cluster.check.check_machines_status()
        regional_cluster.check.check_cluster_nodes(timeout=900)
        regional_cluster.check.check_helmbundles()
        regional_cluster.check.check_persistent_volumes_mounts()

        regional_cluster.store_k8s_artifacts()

        # Check/wait for correct docker service replicas in cluster
        regional_cluster.check.check_actual_expected_docker_services()
        regional_cluster.check.check_k8s_pods()
        regional_cluster.check.check_actual_expected_pods(timeout=1200)
        regional_cluster.check.check_cluster_readiness()
        regional_cluster.provider_resources.save_artifact()
