import pytest
from si_tests import settings


@pytest.mark.parametrize("_", ["CLUSTER_NAME={0}"
                               .format(settings.TARGET_CLUSTER)])
def test_ha_kill_system_proc_in_cont(target_cluster, _):
    """Kill main process in mke system containers
    Precondition - all expected pods and their replicas must be presented
    The following scenario is executed for every node

    Scenario:
        1. SSH to each cluster node 1 by 1
        2. Collect list of container id's
        3. Kill containers 1 by 1
        4. Wait till all containers are running again
        5. Check that all pods are in a correct state

    Expected result - pods and their containers are Running and Ready.
    """
    target_cluster.check.check_ha_kill_system_proc_in_cont()
    target_cluster.check.check_actual_expected_pods()


@pytest.mark.parametrize("_", ["CLUSTER_NAME={0}"
                               .format(settings.TARGET_CLUSTER)])
def test_ha_kill_ucp_cluster_agent(target_cluster, _):
    """Kill ucp-cluster-agent container
    Precondition - all expected pods and their replicas must be presented
    The following scenario is executed for every node

    Scenario:
        1. SSH to each cluster node 1 by 1
        2. Search for ucp-cluster-agent container
        3. Kill ucp-cluster-agent container
        4. Wait till container IS running
        5. Check that all pods are in a correct state
    Expected result - pods and container are Running and Ready.
    """
    target_cluster.check.check_ha_kill_ucp_cluster_agent()
    target_cluster.check.check_actual_expected_pods()


@pytest.mark.parametrize("_", ["CLUSTER_NAME={0}"
                               .format(settings.TARGET_CLUSTER)])
@pytest.mark.skip(reason="PRODX-14150, PRODX-5376")
def test_ha_kill_proc_pods(target_cluster, command, pod_group, _):
    """Kill processed in every container in all cluster pods
    Precondition - all expected pods and their replicas must be presented
    The following scenario is executed for every namespace and
    expected pod entry in get_expected_pods
    Note: 'kill -- - 1' will kill all processes except pid 1

    Scenario:
        1. Iterate by each pod in a group
        2. Iterate by each container in a pod and
           kill processes in the container
        3. Wait till number of replicas will be restored
        4. Check pods and container statuses in this group
           (Running and Ready)
        5. Check all pods phases (must be Running or Completed)
           (in fixture)

    Expected result - pods are recreated, number of replicas is restored.
    """
    target_cluster.check.check_ha_kill_proc_pods(pod_group, command)


@pytest.mark.parametrize("_", ["CLUSTER_NAME={0}"
                               .format(settings.TARGET_CLUSTER)])
@pytest.mark.skip()
def test_ha_delete_pods(target_cluster, pod_group, _):
    """Delete all pods in any cluster one by one
     Precondition - all expected pods and their replicas must be presented
     The following scenario is executed for every namespace and
     expected pod entry in get_expected_pods

     Scenario:
         1. Compare actual number of replicas for pod with expected
         2. Iterate by each replica
         3. Delete pod (1 replica)
         4. Wait till number of replicas will be restored
         5. Check pods statuses in this group (Running and Ready)

     Expected result - pods are recreated, number of replicas is restored.
     """
    target_cluster.check.check_ha_delete_pods(pod_group)


@pytest.mark.parametrize("_", ["CLUSTER_NAME={0}"
                               .format(settings.TARGET_CLUSTER)])
@pytest.mark.skip(reason="ENGORC-7836")
def test_ha_kill_containerd_shim(target_cluster, _):
    """Kill all containerd-shim processes on every cluster node
    Precondition - all expected pods and their replicas must be presented
    The following scenario is executed for every node

    Scenario:
        1. SSH to node and run kill command for containerd-shim processes
        2. Wait
        3. Check that all pods are Running and Ready

    Expected result - pods and their containers are Running and Ready.
    Number of replicas are the same.
    """
    target_cluster.check.check_ha_kill_containerd_shim()
    target_cluster.check_actual_expected_pods()


@pytest.mark.parametrize("_", ["CLUSTER_NAME={0}"
                               .format(settings.TARGET_CLUSTER)])
@pytest.mark.skip(reason="ENGORC-7837")
def test_ha_stop_containerd(target_cluster, _):
    """Stop containerd service on every cluster node (one at a time)
    Precondition - all expected pods and their replicas must be presented
    The following scenario is executed for every node

    Scenario:
        1. SSH to node
        2. Get number of containerd-shim processes
        3. Stop containerd service and make sure it is stopped
        4. Get number of containerd-shim processes and
           compare with previous value (must be the same)
        5. Check that all pods are Running and Ready
        6. Start containerd service

    Expected result - containerd-shim processes are running independently
    from containerd service and all containers in the pods are
    still operational even if containerd is down.
    """
    target_cluster.check.check_ha_stop_containerd()
    target_cluster.check_actual_expected_pods()
