import time

import exec_helpers
import yaml

from si_tests import logger, settings
from si_tests.utils import utils, waiters, templates

LOG = logger.logger


def test_add_bm_ceph_node(kaas_manager):
    """Add one more ceph node."""

    # ns and cluster used for access mgmt cluster
    cluster = kaas_manager.get_mgmt_cluster()
    # managed_ns and managed_cluster used for child cluster
    managed_ns = kaas_manager.get_namespace(settings.TARGET_NAMESPACE)
    managed_cluster = managed_ns.get_cluster(settings.TARGET_CLUSTER)

    cmd_status = ['/bin/sh', '-c', 'ceph -s -f json']
    cmd_osd_tree = ['/bin/sh', '-c', 'ceph osd tree -f json']
    ceph_tools_pod = managed_cluster.get_ceph_tool_pod()

    # Get ssh key
    public_key_name = "{cluster_name}-key".format(
        cluster_name=settings.TARGET_CLUSTER)
    LOG.info("Public key name - {public_key_name}".format(
        public_key_name=public_key_name))
    managed_ns.get_publickey(public_key_name)

    mgmt_version = \
        kaas_manager.get_mgmt_cluster().spec['providerSpec']['value']['kaas'][
            'release']
    # Gather nodes information from yaml file
    child_data = yaml.safe_load(
        templates.render_template(settings.BM_CHILD_SETTINGS_YAML_PATH,
                                  options=dict(mgmt_version=mgmt_version)))[0]
    bm_hosts_data = child_data['nodes']
    managed_ns.create_bm_statics(child_data)

    worker_scale = []
    for node in bm_hosts_data:
        if 'nodeforscale' in node.get('si_roles', []):
            cred_name = node['name'] + '-cred'
            # Dirty hack, for not copy-paste bmh_name|cred_name across whole
            # test.
            bmh_name = utils.render_bmh_name(node['name'],
                                             settings.TARGET_CLUSTER,
                                             node.get('bootUEFI', True),
                                             node['bmh_labels'],
                                             si_roles=node.get('si_roles',
                                                               False))
            _node = node.copy()
            _node.update({'bmh_name': bmh_name,
                          'cred_name': cred_name})
            worker_scale.append(_node)
            if not kaas_manager.api.secrets.present(
                    name=cred_name,
                    namespace=settings.TARGET_NAMESPACE):
                secret_data = {
                    "username": node.get('ipmi', {}).get('username'),
                    "password": node.get('ipmi', {}).get('password')
                }
                managed_ns.create_secret(
                    name=cred_name, data=secret_data,
                    region=settings.TARGET_REGION,
                    provider="baremetal")

    assert len(worker_scale) >= 1, "BMH with si_roles " \
                                   "'nodeforscale' not found"

    LOG.info("worker_scale: {0}".format(worker_scale))
    node = worker_scale[0]
    bmh = managed_ns.get_baremetalhosts()
    # for case when node was added as compute and then was deleted
    # but bmh still present we can reuse bmh without creation
    existing_bmh = [i for i in bmh if "nodeforscale" in i.name]
    if existing_bmh:
        managed_ns.wait_baremetalhosts_statuses(
            nodes=bmh_name,
            wait_status='ready',
            retries=10,
            interval=30)
    else:
        managed_ns.create_baremetalhost(
            bmh_name=bmh_name,
            bmh_secret=node['cred_name'],
            bmh_mac=node['networks'][0]['mac'],
            bmh_ipmi=node['ipmi']['ipmi_ip'],
            hardwareProfile=node.get('hardwareProfile', False),
            labels=node['bmh_labels'],
            bootUEFI=node.get('bootUEFI', True))
        LOG.info(f"New BMH nodes:\n{bmh_name}")

    managed_ns.wait_baremetalhosts_statuses(
        nodes=bmh_name,
        wait_status='ready',
        retries=20,
        interval=60)

    # Create worker machine
    LOG.info('Create BareMetal machine')
    custom_bmhp = False
    if node.get('bmh_profile'):
        custom_bmhp = {
            'namespace': settings.TARGET_NAMESPACE,
            'name': node['bmh_profile']
        }
        ceph_config = node['ceph_cluster_node']
        disk_count = len(ceph_config['storageDevices'])
    new_machine = managed_cluster.create_baremetal_machine(
            genname=node['bmh_name'],
            node_pubkey_name=public_key_name,
            matchlabels={'kaas.mirantis.com/'
                         'baremetalhost-id':
                             node['bmh_labels']
                             ['kaas.mirantis.com/baremetalhost-id']},
            baremetalhostprofile=custom_bmhp,
            l2TemplateSelector=node.get('l2TemplateSelector', dict()),
            labels=node['machine_labels'],
            node_labels=node.get('node_labels', {})
        )

    managed_ns.wait_baremetalhosts_statuses(
        nodes=bmh_name,
        wait_status='provisioned',
        retries=30,
        interval=60)

    managed_cluster.check.check_machines_status()

    # enable automated management of Ceph OSDs
    enable_ceph_lcm = {"spec": {
            "cephClusterSpec": {
                "manageOsds": True
            }
    }}
    ceph_crd = cluster.get_ceph_data(
        namespace=settings.TARGET_NAMESPACE)
    cluster.patch_ceph_data(data=enable_ceph_lcm,
                            crd=ceph_crd)
    add_ceph_node = {
          "spec": {
            "cephClusterSpec": {
              "nodes": {
                new_machine.name: ceph_config
              }
            }
          }
        }
    LOG.info("Cephcluster CRD will be patched with next data:{}{}".format(
        "\n", yaml.dump(add_ceph_node)))
    # Check if HEALTH_OK before patching
    LOG.info("Check if HEALTH_OK before patching CRD")
    ceph_health_timeout = 2400
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)
    LOG.info("Ceph health is HEALTH_OK. Continue...")
    # Get osds num before patch and calculate num after
    num_osds_before_patch = int(yaml.safe_load(ceph_tools_pod.exec(
        cmd_status)).get('osdmap').get('num_osds'))
    num_osds_after_patch = num_osds_before_patch + disk_count
    LOG.info("num_osds_before_patch: {}".format(num_osds_before_patch))
    LOG.info("num_osds_after_patch: {}".format(num_osds_after_patch))
    LOG.info("Patching miraceph cephcluster CRD")
    cluster.patch_ceph_data(data=add_ceph_node,
                            crd=ceph_crd)
    time.sleep(5)
    # Wait for num osds is inceased depending on disks count
    LOG.info("Waiting for OSDs number is increased in ceph cluster")
    waiters.wait(
        lambda: yaml.safe_load(ceph_tools_pod.exec(cmd_status)).get(
            'osdmap').get('num_osds') == num_osds_after_patch,
        timeout=ceph_health_timeout, interval=30,
        timeout_msg="OSDs number doesn't match expected number after {} sec. "
                    "Current osd number is {}. But shoud be {}".format(
            ceph_health_timeout,
            yaml.safe_load(ceph_tools_pod.exec(cmd_status)).get(
                'osdmap').get(
                'num_osds'), num_osds_after_patch))

    # Wait for all osds are UP in cluster
    LOG.info("Waiting for all OSDs are UP in ceph cluster")
    waiters.wait(
        lambda: yaml.safe_load(ceph_tools_pod.exec(cmd_status)).get(
            'osdmap').get(
            'num_up_osds') == num_osds_after_patch,
        timeout=ceph_health_timeout, interval=30,
        timeout_msg="Some osds are not UP after {} sec. "
                    "Current osds UP {}. But shoud be {}".format(
            ceph_health_timeout,
            yaml.safe_load(ceph_tools_pod.exec(cmd_status)).get(
                'osdmap').get(
                'num_up_osds'), num_osds_after_patch))

    # Wait for HEALTH_OK after osds were added
    LOG.info("Final check ceph cluster status")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)
    managed_cluster.check.check_ceph_pvc()
    LOG.info("Ceph cluster is healthy. Congratulations!")

    osd_tree_data = yaml.safe_load(ceph_tools_pod.exec(cmd_osd_tree))
    osds = [node for node in osd_tree_data.get('nodes') if
            node.get('type') == 'osd']
    osds_dict = {}
    for osd in osds:
        osds_dict[osd.get('name')] = {'status': osd.get('status'),
                                      'reweight': osd.get('reweight')}
    LOG.info("Ceph osds info:{}{}".format(
        "\n", yaml.dump(osds_dict)))


def test_replace_failed_osd_disk(kaas_manager):
    """This test emulates disk crashing and replacing

       1. Get information about ceph cluster using ceph tools pod.
       2. Define node and disk for tests
       3. Emulate crash
       4. Check required signals (WARNING,RECENT_CRASH,OSD_DOWN)
       4. Make all required steps according do docs
       4. Verify that ceph cluster is healthy after all operations
    """

    cluster = kaas_manager.get_mgmt_cluster()
    managed_ns = kaas_manager.get_namespace(settings.TARGET_NAMESPACE)
    managed_cluster = managed_ns.get_cluster(settings.TARGET_CLUSTER)

    cmd_status = ['/bin/sh', '-c', 'ceph -s -f json']
    cmd_osd_tree = ['/bin/sh', '-c', 'ceph osd tree -f json']
    archive_cmd = ['/bin/sh', '-c', 'ceph crash archive-all']
    ceph_tools_pod = managed_cluster.get_ceph_tool_pod()
    ceph_operator_pod = managed_cluster.get_ceph_operator_pod()
    # Determine node for working with disk
    nodes = managed_cluster.get_machines()
    target = [i for i in nodes if "nodeforscale" in i.name]
    target_node = target[0]
    ceph_crd = cluster.get_ceph_data(
        namespace=settings.TARGET_NAMESPACE)
    ceph_crd = ceph_crd.read()
    node = ceph_crd.spec['cephClusterSpec']['nodes'][target_node.name]
    failed_disk_name = node['storageDevices'][0]['name']
    failed_disk_node_name = target_node.name
    failed_disk_node_ip = target_node.public_ip
    LOG.info("Working with disk {disk} on node {node}.".format(
        disk=failed_disk_name, node=failed_disk_node_name))
    ceph_health_timeout = 2400
    keys = utils.load_keyfile(settings.KAAS_CHILD_CLUSTER_PRIVATE_KEY_FILE)
    pkey = utils.get_rsa_key(keys['private'])
    LOG.info("Verify that all osds are UP")
    osds_info = yaml.safe_load(ceph_tools_pod.exec(cmd_osd_tree))
    osds = [node for node in osds_info.get('nodes') if
            node.get('type') == 'osd']
    nodes = [node for node in osds_info.get('nodes') if
             node.get('type') == 'host']
    osds_dict = {}
    nodes_dict = {}
    for osd in osds:
        osds_dict[osd.get('name')] = {'status': osd.get('status'),
                                      'reweight': osd.get('reweight')}
    for node in nodes:
        nodes_dict[node.get('name')] = {'osds': [
            'osd.' + str(child) for child in node.get('children')]}
    LOG.info("Ceph nodes info:{}{}".format("\n", yaml.dump(nodes_dict)))
    LOG.info("Ceph osds info:{}{}".format("\n", yaml.dump(osds_dict)))

    # Here we will simulate disk crash
    LOG.info("Cleaning {disk} disk on node {node} "
             "and remove it from system".format(
                 disk=failed_disk_name, node=failed_disk_node_name))
    LOG.info(f"Going to ssh on {failed_disk_node_ip}")
    auth = exec_helpers.SSHAuth(username='mcc-user',
                                password='', key=pkey)

    ssh = exec_helpers.SSHClient(
        host=failed_disk_node_ip, port=22, auth=auth)
    ssh.logger.addHandler(logger.console)

    ssh.sudo_mode = True
    ssh.check_call(
        "dd if=/dev/zero of=/dev/{} bs=100M count=10".format(
            failed_disk_name))
    ssh.check_call(
        "echo '1' > /sys/block/{}/device/delete".format(
            failed_disk_name))

    # Wait for HEALTH_WARN
    LOG.info("Waiting for WARNING signal after disk was crashed.")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout, expected_status='HEALTH_WARN')

    # Check that only 1 osd is down and belongs to choosen node
    osds_info = yaml.safe_load(ceph_tools_pod.exec(cmd_osd_tree))
    osds_down = [node.get('name') for node in osds_info.get(
        'nodes') if node.get('type') == 'osd' and node.get(
        'status') == 'down']
    assert len(osds_down) == 1, ("More that 1 osds are down: {}".format(
        osds_down))
    osds = [node for node in osds_info.get('nodes') if
            node.get('type') == 'osd']
    for osd in osds:
        osds_dict[osd.get('name')] = {'status': osd.get('status'),
                                      'reweight': osd.get('reweight')}
    LOG.info("Current osds stats: {}{}".format("\n", yaml.dump(osds_dict)))

    # Now we need to wait until information about crash appears in ceph status
    LOG.info("Waiting for information about "
             "recent crash appears in ceph status")
    waiters.wait(
        lambda: 'RECENT_CRASH' in yaml.safe_load(
            ceph_tools_pod.exec(cmd_status)).get('health', {}).get(
            'checks'), timeout=ceph_health_timeout, interval=30)

    LOG.info("Crash INFO: {}".format(yaml.safe_load(
        ceph_tools_pod.exec(cmd_status)).get('health', {}).get(
        'checks')))

    LOG.info("Archiving information about crash")
    ceph_tools_pod.exec(archive_cmd)

    # Wait for rebalancing and HEALTH_OK.
    LOG.info("Waiting for data rebalanced and HEALTH is OK")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)

    osd_tree_data = yaml.safe_load(ceph_tools_pod.exec(cmd_osd_tree))
    LOG.info("osd tree data: {}".format(osd_tree_data))

    # Define osd num which is down after disk crash
    LOG.info("Define osd ID for further work")
    osd_num = osds_down[0]
    LOG.info("OSD ID is {}".format(osd_num))
    osd_id_number = osd_num.split('.')[1]

    # Remove the Ceph OSD deployment from the managed cluster:
    deployment = managed_cluster.k8sclient.deployments.get(
        name="rook-ceph-osd-{}".format(osd_id_number),
        namespace=settings.ROOK_CEPH_NS)
    LOG.info("Deleting deployment rook-ceph-osd-{}".format(osd_id_number))
    deployment.delete()
    LOG.info("Done")

    # Remove the failed Ceph OSD from the Ceph cluster
    purge_cmd = ['/bin/sh', '-c', 'ceph osd purge {}'.format(
        osd_num)]
    del_auth_cmd = ['/bin/sh', '-c', 'ceph auth del {}'.format(
        osd_num)]
    LOG.info("Purging {}".format(osd_num))
    ceph_tools_pod.exec(purge_cmd)
    LOG.info("Remove auth for purged osd")
    ceph_tools_pod.exec(del_auth_cmd)

    LOG.info("Waiting for data rebalanced and HEALTH is OK")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)

    LOG.info("Rebooting node {}".format(failed_disk_node_name))
    ssh.reconnect()
    ssh.check_call("iptables -A INPUT -p tcp --dport 22 -j REJECT; "
                   "/sbin/shutdown -r -f now &>/dev/null & exit")
    LOG.info("Node is being rebooted.")
    waiters.wait_tcp(failed_disk_node_ip, port=22, timeout=600)
    LOG.info("Node is rebooted succesfully. Establishing new connection...")
    ssh = exec_helpers.SSHClient(host=failed_disk_node_ip, port=22, auth=auth)
    ssh.logger.addHandler(logger.console)

    LOG.info("Restarting ceph-operator")
    ceph_operator_pod.delete()

    # Wait for new osd appears in cluster
    LOG.info("Waiting for {} appears in ceph cluster".format(
        osd_num))
    waiters.wait(
        lambda: osd_num in [osd.get('name') for osd in yaml.safe_load(
            ceph_tools_pod.exec(cmd_osd_tree)).get('nodes')],
        timeout=ceph_health_timeout, interval=30,
        timeout_msg="{} is not existed in ceph cluster "
                    "after {} sec".format(osd_num, ceph_health_timeout))

    LOG.info("OSD {} existed in ceph cluster. Continue".format(osd_num))
    osd_pod = managed_cluster.k8sclient.pods.list(
        namespace=settings.ROOK_CEPH_NS,
        name_prefix='rook-ceph-osd-{}'.format(osd_id_number))[0]
    osd_pod_data = osd_pod.data
    osd_pod_name = osd_pod_data.get('metadata').get('name')
    LOG.info("Waiting for pod {} is Running".format(osd_pod_name))
    try:
        osd_pod.wait_ready(timeout=300, interval=30)
    except Exception:
        # https://docs.mirantis.com/container-cloud/latest/operations-guide/manage-ceph/replace-failed-osd.html # noqa
        LOG.info(f"Pod {osd_pod_name} was not able to start. "
                 "Will proceed with recovering operation")
        LOG.info(f"Getting keyring data for pod {osd_pod_name}")
        src_keyring_path = '/tmp/key'
        dst_keyring_path = '/tmp/key'
        osd_pod_volumes = [v for v in
                           osd_pod_data.get('spec').get('volumes', [])]
        block_device_path = [d.get('host_path').get('path') for d in
                             osd_pod_volumes if
                             d.get('name') == 'activate-osd'][0]
        keyring_path = block_device_path + '/keyring'
        keyring_data = ssh.check_call(f'cat {keyring_path}')
        std_lines = keyring_data.stdout_lines
        keyring = [line.split('key =')[1].strip() for line in std_lines
                   if 'key =' in line]
        if keyring:
            keyring = keyring[0]
            LOG.info(f"Keyring data found. Key: {keyring}")
        else:
            raise Exception(f"Keyring not found for pod {osd_pod_name}")
        with open(src_keyring_path, 'w') as f:
            f.write(f'[osd.{osd_id_number}]' + '\n' +
                    f'key = {keyring}' + '\n' +
                    'caps mgr = \"allow profile osd\"' + '\n' +
                    'caps mon = \"allow profile osd\"' + '\n' +
                    'caps osd = \"allow *\"')

        LOG.info("Writing keyring data to ceph-tools-pod for further usage")
        with open(src_keyring_path, 'r') as f:
            commands = ['/bin/sh',
                        '-c',
                        f'cat <<EOF > {dst_keyring_path} \n{f.read()}\nEOF']
            ceph_tools_pod.exec(commands)
        LOG.info("Done.")
        LOG.info(f"Importing new keyring data for {osd_pod_name} pod")
        auth_import_cmd = ['/bin/sh',
                           '-c',
                           f'ceph auth import -i {dst_keyring_path}']
        LOG.info(ceph_tools_pod.exec(auth_import_cmd))
        LOG.info(f"Restarting {osd_pod_name} pod")
        osd_pod.delete()
        LOG.info('Waiting for new osd pod starts')
        waiters.wait_pass(lambda: managed_cluster.k8sclient.pods.list(
            namespace=settings.ROOK_CEPH_NS,
            name_prefix='rook-ceph-osd-{}'.format(osd_id_number))[0])
        new_osd_pod = managed_cluster.k8sclient.pods.list(
            namespace=settings.ROOK_CEPH_NS,
            name_prefix='rook-ceph-osd-{}'.format(osd_id_number))[0]
        new_osd_pod_name = new_osd_pod.data.get('metadata').get('name')
        LOG.info(f"New osd pod name: {new_osd_pod_name}. "
                 "Waiting for Running state")
        new_osd_pod.wait_ready(timeout=600, interval=60)
    LOG.info("Waiting for {} become UP status".format(osd_num))
    waiters.wait(
        lambda: [node.get('status') for node in yaml.safe_load(
            ceph_tools_pod.exec(cmd_osd_tree)).get('nodes') if node.get(
                'name') == osd_num][0] == "up",
        timeout=ceph_health_timeout, interval=30,
        timeout_msg="{} is failed to bacome UP status "
                    "after {} sec".format(osd_num, ceph_health_timeout))

    LOG.info("Waiting for data rebalanced and HEALTH is OK")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)
    managed_cluster.check.check_ceph_pvc()
    LOG.info("Everything seems to be fine. Congratulations!")

    osd_tree_data = yaml.safe_load(ceph_tools_pod.exec(cmd_osd_tree))
    osds = [node for node in osd_tree_data.get('nodes') if
            node.get('type') == 'osd']
    osds_dict = {}
    for osd in osds:
        osds_dict[osd.get('name')] = {'status': osd.get('status'),
                                      'reweight': osd.get('reweight')}
    LOG.info("Ceph osds info:{}{}".format("\n", yaml.dump(osds_dict)))


def test_remove_ceph_osd_node(kaas_manager):
    """This test removes osd node from already created ceph cluster.

       1. Get information about ceph cluster using ceph tools pod.
       2. Define node for downscaling
       3. Remove node from ceph cluster
       4. Wait ceph health OK
       5. Delete node from machines
       6. Delete bmh
       7. Wait ceph health OK
    """

    ceph_health_timeout = 2400
    cluster = kaas_manager.get_mgmt_cluster()
    managed_ns = kaas_manager.get_namespace(settings.TARGET_NAMESPACE)
    managed_cluster = managed_ns.get_cluster(settings.TARGET_CLUSTER)

    cmd_status = ['/bin/sh', '-c', 'ceph -s -f json']
    cmd_osd_tree = ['/bin/sh', '-c', 'ceph osd tree -f json']
    ceph_tools_pod = managed_cluster.get_ceph_tool_pod()

    machines_for_delete = [machine for machine in managed_ns.get_machines()
                           if 'nodeforscale' in machine.name]
    machine = machines_for_delete[0]

    ceph_crd = cluster.get_ceph_data(
        namespace=settings.TARGET_NAMESPACE)
    new_crd = ceph_crd.read()
    disk_count = len(
        new_crd.spec['cephClusterSpec']
        ['nodes'][machine.name]['storageDevices'])
    new_crd.spec['cephClusterSpec']['nodes'].pop(machine.name)
    LOG.info("Check if HEALTH_OK before patching CRD")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)
    LOG.info("Ceph health is HEALTH_OK. Continue...")
    # Get osds num before patch and calculate num after
    num_osds_before_patch = int(yaml.safe_load(
        ceph_tools_pod.exec(cmd_status)).get(
        'osdmap').get('num_osds'))
    LOG.info(f"OSDs before patch: {num_osds_before_patch}")
    num_osds_after_patch = num_osds_before_patch - disk_count
    LOG.info(f"OSDs after patch: {num_osds_after_patch}")
    LOG.info("Machine for delete: {}".format(machine.name))
    LOG.info("Patching miraceph cephcluster CRD")
    cluster.replace_ceph_data(data=new_crd, crd=ceph_crd)
    time.sleep(5)
    LOG.info("Waiting for OSDs number is decreased in ceph cluster")
    ceph_tools_pod = managed_cluster.get_ceph_tool_pod()
    waiters.wait(
        lambda: yaml.safe_load(ceph_tools_pod.exec(cmd_status)).get(
            'osdmap').get(
            'num_osds') == num_osds_after_patch,
        timeout=ceph_health_timeout, interval=30,
        timeout_msg="OSDs number doesn't match expected number after {} sec. "
                    "Current osd number is {}. But shoud be {}".format(
            ceph_health_timeout,
            yaml.safe_load(ceph_tools_pod.exec(cmd_status)).get(
                'osdmap').get(
                'num_osds'), num_osds_after_patch))

    # Wait for HEALTH_OK after osds were deleted
    LOG.info("Check ceph cluster status")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)
    LOG.info("Ceph cluster is healthy. Congratulations!")

    osd_tree_data = yaml.safe_load(ceph_tools_pod.exec(cmd_osd_tree))
    osds = [node for node in osd_tree_data.get('nodes') if
            node.get('type') == 'osd']
    osds_dict = {}
    for osd in osds:
        osds_dict[osd.get('name')] = {'status': osd.get('status'),
                                      'reweight': osd.get('reweight')}
    LOG.info("Ceph osds info:{}{}".format(
        "\n", yaml.dump(osds_dict)))
    LOG.info("Waiting for osd pods number is decreased "
             "according to disks count")
    waiters.wait(lambda: len(
        [pod for pod in managed_cluster.k8sclient.pods.list(
            namespace='rook-ceph') if 'rook-ceph-osd-' in pod.name
         and 'osd-prepare' not in pod.name]) == num_osds_after_patch,
        timeout=ceph_health_timeout, interval=30,
        timeout_msg="osd pods number is not equal to osd disks count "
                    "in ceph cluster. Current osd pods number is {}. "
                    "But shoud be {}".format(
            len([pod for pod in managed_cluster.k8sclient.pods.list(
                namespace='rook-ceph') if 'rook-ceph-osd-' in pod.name]),
            num_osds_after_patch))
    LOG.info("Delete machine")
    machine.delete()
    bmh_for_delete = [bmh for bmh in managed_ns.get_baremetalhosts()
                      if 'nodeforscale' in bmh.name]
    bmh = bmh_for_delete[0]
    managed_cluster.wait_machine_deletion(machine.name)
    managed_ns.wait_baremetalhosts_statuses(
        nodes=bmh.name,
        wait_status='ready',
        retries=30,
        interval=60)
    LOG.info("Deleting BMH {}".format(bmh.name))
    bmh.delete()
    managed_ns.wait_baremetalhost_deletion(bmh.name)
    LOG.info("Next BMHs were deleted succesfully: {}".format(
        bmh_for_delete))
    time.sleep(5)

    # Wait for HEALTH_OK after osds were added
    LOG.info("Check ceph cluster status")
    managed_cluster.check.wait_ceph_health_status(
        timeout=ceph_health_timeout)
    managed_cluster.check.check_ceph_pvc()
    LOG.info("Ceph cluster is healthy. Congratulations!")
